#############
# $Id: fastq_by_tag.pm 12510 2011-02-01 15:49:31Z ajb $
# Created By: ajb
# Last Maintained By: $Author: ajb $
# Created On: 2010-01-22
# Last Changed On: $Date: 2011-02-01 15:49:31 +0000 (Tue, 01 Feb 2011) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-pipeline/tags/release-24.2/lib/npg_pipeline/archive/file/generation/fastq_by_tag.pm $

package npg_pipeline::archive::file::generation::fastq_by_tag;
use Moose;
use MooseX::InsideOut;
use Carp;
use English qw{-no_match_vars};
use Readonly;
use File::Basename;

extends q{npg_pipeline::archive::file::generation};

Readonly::Scalar our $VERSION  => do { my ($r) = q$LastChangedRevision: 12510 $ =~ /(\d+)/mxs; $r; };

sub generate {
  my ($self, $arg_refs) = @_;

  my $id_run = $self->id_run();

  $self->log(qq{Creating Jobs to split fastq files by tag for $id_run});

  my @positions = $self->positions($arg_refs);

  my @job_ids;

  if (! $self->is_indexed()) {
    $self->log(qq{Run $id_run is not multiplex run and no need to split});
    return;
  }

  my $href = $self->lanes_by_split_nonsplit( @positions );

  my $output_dir = $self->archive_path() . q{/lane};

  foreach my $position ( @{ $href->{indexed_lanes} } ) {

    $arg_refs->{position} = $position;

    my $lane_output_dir = $output_dir . $position;
    if ( ! -d $lane_output_dir ) {
       $self->log( qq{creating $lane_output_dir} );
       my $rc = `mkdir -p $lane_output_dir`;
       if ( $CHILD_ERROR ) {
         croak qq{could not create $lane_output_dir\n\t$rc};
       }
    }

  }

  my $position = $self->lsb_jobindex();

  $output_dir .= $position;
  $arg_refs->{output_dir} = $output_dir;

  # submits a job (in an array) foreach multiplex lane
  if ( scalar @{ $href->{indexed_lanes} } ) {
    $arg_refs->{array_string} = $self->create_array_string( @{ $href->{indexed_lanes} } );

    foreach my $fastq ( @{ $self->_fastq_files_per_lane( $position ) } ) {
      $arg_refs->{fastq}    = $fastq;
      $arg_refs->{split_type} = q{standard};
      my $job_sub = $self->_generate_bsub_command( $arg_refs );
      push @job_ids, $self->submit_bsub_command( $job_sub );
    }

  }

  # submits a human and a non_human job (in an array) foreach multiplex lane which should have non_consented human split out
  if ( scalar @{ $href->{split_nc_mp_lanes} } ) {

    $arg_refs->{array_string} = $self->create_array_string( @{ $href->{split_nc_mp_lanes} } );

    my $fastq_files_per_lane = {};
    $fastq_files_per_lane->{human}    = $self->_fastq_files_per_lane( $position, q{human} );
    $fastq_files_per_lane->{nonhuman} = $self->_fastq_files_per_lane( $position, q{nonhuman} );

    foreach my $split_type ( sort keys %{ $fastq_files_per_lane } ) {
      $arg_refs->{split_type} = $split_type;

      foreach my $fastq ( @{ $fastq_files_per_lane->{$split_type} } ) {
        $arg_refs->{fastq} = $fastq;
        my $job_sub = $self->_generate_bsub_command( $arg_refs );
        push @job_ids, $self->submit_bsub_command( $job_sub );
      }

    }

  }

  return @job_ids;
}

###############
# responsible for generating the bsub command to be executed for one lane one fastq file
sub _generate_bsub_command {
  my ($self, $arg_refs) = @_;

  my $split_type              = $arg_refs->{split_type};
  my $fastq_file              = $arg_refs->{fastq};
  my $required_job_completion = $arg_refs->{required_job_completion};
  my $output_dir              = $arg_refs->{output_dir};

  my ($fastq_name, $dir, $suffix) =  fileparse($fastq_file, qw(.fastq));

  my $timestamp = $self->timestamp();

  my $script_bin = $self->internal_script_bin();

  my $end = $fastq_file;
  $end =~ s/.fastq//gxms;
  $end =~ s/_${split_type}//xms;
  ($end) = $end =~ /_([a-z0-9]+)\z/xms;

  my $job_name = $self->external_script_names_conf()->{split_fastq_by_tag_script} . q{_} . $self->id_run() . q{_end} . $end . q{_} . $split_type . q{_} . $timestamp;
  my $replacement = $job_name . q{.%I.%J};
  $self->log($fastq_file);
  $job_name .= $arg_refs->{array_string};

  my $outfile = $self->make_log_dir( $self->archive_path() ) . q{/} . $self->bsub_output_file();
  $outfile =~ s/substitute_this/$replacement/xms;

  my $job_command = qq{$script_bin/} . $self->external_script_names_conf()->{split_fastq_by_tag_script} . qq{ --fastq $fastq_file --output_dir $output_dir};

  my $job_sub = q{bsub -q } . $self->lsf_queue() . q{ };
  $job_sub .=  ( $self->fs_resource_string( {
    counter_slots_per_job => 1,
  } ) ) . q{ };
  $job_sub .= qq{$required_job_completion -J $job_name -o $outfile '$job_command'};

  if ($self->verbose()) { $self->log($job_sub); }
  return $job_sub;
}

sub _fastq_files_per_lane {
  my ($self, $position, $split_type) = @_;

  my $archive_path = $self->archive_path();

  my @fastq_filesnames = $self->generate_fastq_and_check_name_parts( { lane => $position } );

  my @fastq_files = ();

  foreach my $part_fastq_name ( @fastq_filesnames ) {

    if ( $part_fastq_name =~ /_t$/mxs ) {
      next;
    }

    my $full_fastq_name = $archive_path . q{/} . $self->id_run.q{_} . $part_fastq_name;
    if ( $split_type ) {
      $full_fastq_name .= q{_} . $split_type;
    }
    $full_fastq_name .= q{.fastq};

    push @fastq_files, $full_fastq_name;
  }
  return \@fastq_files;
}

no Moose;
__PACKAGE__->meta->make_immutable;
1;
__END__

=head1 NAME

npg_pipeline::archive::file::generation::fastq_by_tag

=head1 VERSION

$LastChangedRevision: 12510 $

=head1 SYNOPSIS

  my $oAfgfq = npg_pipeline::archive::file::generation::fastq_by_tag->new({
    run_folder => $sRunFolder,
  });

=head1 DESCRIPTION

Object module which knows how to construct and submits the command line to LSF for creating bam file.

=head1 SUBROUTINES/METHODS

=head2 generate - generates the bsub jobs and submits them for spliting the fastq files by tag, returning an array of job_ids.

  my @job_ids = $oAfgfq->generate({
    required_job_completion} => q{-w (123 && 321)};
    timestamp                => q{20090709-123456};
    solexa_bin               => q{/software/solexa/bin};
    id_run                   => 1234;
    BSUB_ERROR_FILE          => q{substitute_this.err};
    BSUB_OUTPUT_FILE         => q{substitute_this.out};
  });

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Carp

=item English -no_match_vars

=item Readonly

=item Moose

=item npg_pipeline::archive::file::generation

=item File::Basename

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: ajb $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Guoying Qi (gq1@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
