#############
# $Id: gcfreq.pm 12510 2011-02-01 15:49:31Z ajb $
# Created By: ajb
# Last Maintained By: $Author: ajb $
# Created On: 2009-07-10
# Last Changed On: $Date: 2011-02-01 15:49:31 +0000 (Tue, 01 Feb 2011) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-pipeline/tags/release-24.2/lib/npg_pipeline/archive/file/generation/gcfreq.pm $

########################
#
# This is no longer generated
# Aim to remove the code completely in Release 9.0
#
########################

package npg_pipeline::archive::file::generation::gcfreq;
use Moose;
use MooseX::InsideOut;
use Carp;
use English qw{-no_match_vars};
use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$LastChangedRevision: 12510 $ =~ /(\d+)/mxs; $r; };

extends q{npg_pipeline::archive::file::generation};

sub generate {
  my ($self, $arg_refs) = @_;

  my $id_run = $self->id_run();

  $self->log(qq{Creating Jobs to create gcfreq files for $id_run});

  my @job_ids;

  my @gcfreq_file_names = $self->generate_fastq_and_check_name_parts($arg_refs);

  foreach my $file_name (@gcfreq_file_names) {
    $arg_refs->{file_name} = $file_name;
    my $job_sub = $self->_generate_bsub_command($arg_refs);
    push @job_ids, $self->submit_bsub_command($job_sub);
  }

  return @job_ids;
}

###############
# responsible for generating the bsub command to be executed
sub _generate_bsub_command {
  my ($self, $arg_refs) = @_;

  my $file_name = $arg_refs->{file_name};
  my $required_job_completion = $arg_refs->{required_job_completion};
  my $timestamp = $self->timestamp();
  my $script_bin = $self->internal_script_bin();
  my $id_run = $self->id_run();

  my $outfile = $self->make_log_dir( $self->archive_path() ) . q{/} . $self->bsub_output_file();
  my $replacement = q{gcfreq_creation_} . $file_name . q{_} . $timestamp;
  my $job_name = q{gcfreq_creation_} . $id_run . q{_} . $file_name . q{_} . $timestamp;
  $outfile =~ s/substitute_this/$replacement/xms;

  my $fast_q_file = $self->archive_path().q{/}.$self->fastq_file_constructs();
  $fast_q_file =~ s/substitute_this/$file_name/xms;
  my $gcfreq_file = $self->archive_path().q{/}.$self->gcfreq_file_constructs();
  $gcfreq_file =~ s/substitute_this/$file_name/xms;

  my $job_sub = q{bsub -q }. $self->lsf_queue() . qq{ $required_job_completion -J $job_name -o $outfile '$script_bin/} . $self->external_script_names_conf()->{gcfreq_creator_script} . qq{ -q $fast_q_file > $gcfreq_file'};

  if ($self->verbose()) { $self->log($job_sub); }
  return $job_sub;
}

no Moose;
__PACKAGE__->meta->make_immutable;
1;
__END__

=head1 NAME

npg_pipeline::archive::file::generation::gcfreq

=head1 VERSION

$LastChangedRevision: 12510 $

=head1 SYNOPSIS

  my $oAfggcfreq = npg_pipeline::archive::file::generation::gcfreq->new({
    run_folder => $sRunFolder,
  });

=head1 DESCRIPTION

Object module which knows how to construct and submits the command line to LSF for creating gcfreq files.

=head1 SUBROUTINES/METHODS

=head2 generate - generates the bsub jobs and submits them for creating the gcfreq files, returning an array of job_ids.

  my @job_ids = $oAfggcfreq->generate({
    required_job_completion} => q{-w (123 && 321)};
    timestamp                => q{20090709-123456};
    solexa_bin               => q{/software/solexa/bin};
    id_run                   => 1234;
    BSUB_ERROR_FILE          => q{substitute_this.err};
    BSUB_OUTPUT_FILE         => q{substitute_this.out};
  });

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Carp

=item English -no_match_vars

=item Readonly

=item Moose

=item npg_pipeline::archive::file::generation

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: ajb $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Andy Brown (ajb@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
