#############
# $Id: srf.pm 12510 2011-02-01 15:49:31Z ajb $
# Created By: ajb
# Last Maintained By: $Author: ajb $
# Created On: 2009-07-09
# Last Changed On: $Date: 2011-02-01 15:49:31 +0000 (Tue, 01 Feb 2011) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-pipeline/tags/release-24.2/lib/npg_pipeline/archive/file/generation/srf.pm $

package npg_pipeline::archive::file::generation::srf;
use Moose;
use MooseX::InsideOut;
use Carp;
use English qw{-no_match_vars};
use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$LastChangedRevision: 12510 $ =~ /(\d+)/mxs; $r; };
use lib qw{lib /software/solexa/lib/perl5};

extends q{npg_pipeline::archive::file::generation};
with q{npg_pipeline::roles::business::tag_info};

Readonly::Scalar our $SLOTS_PER_JOB        => 3;

sub generate {
  my ($self, $arg_refs) = @_;

  my @job_ids;

  if ( $self->no_srf_generation() ) {
    return @job_ids;
  }

  my $id_run = $self->id_run();

  my @srf_file_names = $self->positions( $arg_refs );

  my $intensity_path = $self->intensity_path();
  my $ipar_ints = scalar glob "$intensity_path/s_*_int.txt.p.gz";

  if ($intensity_path =~ /Intensities/xms || $ipar_ints) {
    $ipar_ints = q{-I};
  } else {
    $ipar_ints = q{};
  }

  $arg_refs->{ipar_ints} = $ipar_ints;

  my $href = $self->lanes_by_split_nonsplit( $self->positions( $arg_refs ) );

  foreach my $type ( qw{ indexed_lanes non_indexed_lanes } ) {

    next if ! scalar @{ $href->{ $type } };

    $arg_refs->{type_of_lanes} = $type;

    $arg_refs->{array_string} = $self->create_array_string( @{ $href->{ $type } } );

    my $job_sub = $self->_generate_srf_bsub_command($arg_refs);
    my $jid = $self->submit_bsub_command($job_sub);

    # srfs need to be indexed, and this needs an extra job due to the way the memory seems to clean
    # up create a dependency on the srf creation being successfully completed, an push back the index
    # completion as the dependency for the next level of submitted jobs
    my ($dependency) = $jid =~ /(\d+)/xms;
    my %arg_refs_for_indexing = %{$arg_refs};
    $arg_refs_for_indexing{required_job_completion} = qq{-w'done($dependency} . q{[*])'};
    $job_sub = $self->_generate_index_bsub_command(\%arg_refs_for_indexing);
    push @job_ids, $self->submit_bsub_command($job_sub);
  }


  return @job_ids;
}

###############
# responsible for generating the bsub command to be executed
sub _generate_srf_bsub_command {
  my ($self, $arg_refs) = @_;

  my $required_job_completion = $arg_refs->{required_job_completion};
  my $timestamp = $self->timestamp();
  my $script_bin = $self->internal_script_bin();
  my $id_run = $self->id_run();
  my $ipar_ints = $arg_refs->{ipar_ints};

  my $bustard_dir = $self->bustard_path();

  my $outfile = $self->make_log_dir( $self->archive_path() ) . q{/} . $self->bsub_output_file();

  my $job_name = q{srf_creation_} . $id_run . q{_} . $timestamp;
  my $replacement = $job_name . q{.%I.%J};
  $job_name .= $arg_refs->{array_string};

  $outfile =~ s/substitute_this/$replacement/xms;

  my $file_name = $self->lsb_jobindex();
  my $qseq_file_string = $self->qseq_file_constructs($file_name);

  if ( ! $qseq_file_string || $qseq_file_string =~ /\A\s*\z/xms ) {
    croak q{No qseq files found which give appropriate match};
  }

  my $srf_file = $self->archive_path() . q{/} . $self->srf_file_constructs;
  $srf_file =~ s/substitute_this/$file_name/xms;

  my $use_bases = $self->corrected_tag_part_use_bases();

  my $job_sub = q{bsub -q } . $self->lsf_queue() . q{ };
  $job_sub .=  ( $self->fs_resource_string( {
    resource_string => q{-R 'select[mem>4000] rusage[mem=4000]' -M4000000}, # whilst 2Gb should be more than enough, requesting 4 ensures that no more than 4 will be launched on any one node, which slows it down
    counter_slots_per_job => $SLOTS_PER_JOB,
  } ) ) . q{ };
  $job_sub .= qq{$required_job_completion -J $job_name -o $outfile '$script_bin/};
  $job_sub .= $self->external_script_names_conf()->{srf_creator_script};
  $job_sub .= qq{ $ipar_ints -b -filter-bad-reads};
  if ( $self->is_indexed() ) {
    if ( $arg_refs->{type_of_lanes} eq q{indexed_lanes} ) {
      $job_sub .= q{ -n %x:%y#%i};
    } else {
      $job_sub .= q{ -n %x:%y};
    }
  }
  $job_sub .= qq{ -bustard-dir $bustard_dir -use_bases  $use_bases -o $srf_file $qseq_file_string'};

  if ($self->verbose()) { $self->log($job_sub); }

  return $job_sub;
}

sub _generate_index_bsub_command {
  my ($self, $arg_refs) = @_;

  return q{} if $self->no_srf_generation();

  my $file_name = $self->lsb_jobindex();
  my $required_job_completion = $arg_refs->{required_job_completion};
  my $timestamp = $self->timestamp();
  my $id_run = $self->id_run();

  my $srf_file = $arg_refs->{srf_file};

  $srf_file ||= $self->archive_path() . q{/} . $self->srf_file_constructs;
  $srf_file =~ s/substitute_this/$file_name/xms;

  my $outfile = $self->make_log_dir( $self->archive_path() ) . q{/} . $self->bsub_output_file();
  my $job_name =  $self->external_script_names_conf()->{srf_indexing_script} . q{_} . $id_run . q{_} . $timestamp;
  my $replacement = $job_name . q{.%I.%J};
  $job_name .= $arg_refs->{array_string};
  $outfile =~ s/substitute_this/$replacement/xms;

  my $job_sub = q{bsub -q } . $self->lsf_queue() . q{ };
  $job_sub .=  ( $self->fs_resource_string( {
    resource_string => q{-R 'select[mem>8000] rusage[mem=8000]' -M8000000}, # upped to 8Gb because HiSeq's need more memory
    counter_slots_per_job => 2,
  } ) ) . q{ };
  $job_sub .= qq{$required_job_completion -J $job_name -o $outfile '} . $self->internal_script_bin() . q{/} . $self->external_script_names_conf()->{srf_indexing_script} . qq{ $srf_file'};

  if ($self->verbose()) { $self->log($job_sub); }

  return $job_sub;
}
no Moose;
__PACKAGE__->meta->make_immutable;
1;
__END__

=head1 NAME

npg_pipeline::archive::file::generation::srf

=head1 VERSION

$LastChangedRevision: 12510 $

=head1 SYNOPSIS

  my $oAfgsrf = npg_pipeline::archive::file::generation::srf->new({
    run_folder => $sRunFolder,
  });

=head1 DESCRIPTION

Object module which knows how to construct and submits the command line to LSF for creating srf files.

=head1 SUBROUTINES/METHODS

=head2 generate - generates the bsub jobs and submits them for creating the srf files, returning an array of job_ids.

  my @job_ids = $oAfgsrf->generate({
    required_job_completion} => q{-w (123 && 321)};
    timestamp                => q{20090709-123456};
    pipeline_bin             => q{/software/pipeline/bin};
    id_run                   => 1234;
    BSUB_ERROR_FILE          => q{substitute_this.err};
    BSUB_OUTPUT_FILE         => q{substitute_this.out};
  });

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Moose

=item Carp

=item English -no_match_vars

=item Readonly

=item npg_pipeline::archive::file::generation

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: ajb $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Andy Brown (ajb@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
#############
# $Id: srf.pm 12510 2011-02-01 15:49:31Z ajb $
# Created By: ajb
# Last Maintained By: $Author: ajb $
# Created On: 2009-07-09
# Last Changed On: $Date: 2011-02-01 15:49:31 +0000 (Tue, 01 Feb 2011) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-pipeline/tags/release-24.2/lib/npg_pipeline/archive/file/generation/srf.pm $

package npg_pipeline::archive::file::generation::srf;
use Moose;
use MooseX::InsideOut;
use Carp;
use English qw{-no_match_vars};
use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$LastChangedRevision: 12510 $ =~ /(\d+)/mxs; $r; };
use lib qw{lib /software/solexa/lib/perl5};

extends q{npg_pipeline::archive::file::generation};
with q{npg_pipeline::roles::business::tag_info};

sub generate {
  my ($self, $arg_refs) = @_;

  my $id_run = $self->id_run();

  my @job_ids;

  my @srf_file_names = $self->positions( $arg_refs );

  my $intensity_path = $self->intensity_path();
  my $ipar_ints = scalar glob "$intensity_path/s_*_int.txt.p.gz";

  if ($intensity_path =~ /Intensities/xms || $ipar_ints) {
    $ipar_ints = q{-I};
  } else {
    $ipar_ints = q{};
  }

  $arg_refs->{ipar_ints} = $ipar_ints;

  my $href = $self->lanes_by_split_nonsplit( $self->positions( $arg_refs ) );

  my @srf_creation_jids;
  foreach my $type ( qw{ indexed_lanes non_indexed_lanes } ) {

    next if ! scalar @{ $href->{ $type } };

    $arg_refs->{type_of_lanes} = $type;

    $arg_refs->{array_string} = $self->create_array_string( @{ $href->{ $type } } );

    my $job_sub = $self->_generate_bsub_command($arg_refs);
    my $jid = $self->submit_bsub_command($job_sub);

    # srfs need to be indexed, and this needs an extra job due to the way the memory seems to clean
    # up create a dependency on the srf creation being successfully completed, an push back the index
    # completion as the dependency for the next level of submitted jobs
    my ($dependency) = $jid =~ /(\d+)/xms;
    my %arg_refs_for_indexing = %{$arg_refs};
    $arg_refs_for_indexing{required_job_completion} = qq{-w'done($dependency} . q{[*])'};
    $job_sub = $self->_generate_index_bsub_command(\%arg_refs_for_indexing);
    push @job_ids, $self->submit_bsub_command($job_sub);
  }


  return @job_ids;
}

###############
# responsible for generating the bsub command to be executed
sub _generate_srf_bsub_command {
  my ($self, $arg_refs) = @_;

  my $required_job_completion = $arg_refs->{required_job_completion};
  my $timestamp = $self->timestamp();
  my $script_bin = $self->internal_script_bin();
  my $id_run = $self->id_run();
  my $ipar_ints = $arg_refs->{ipar_ints};

  my $bustard_dir = $self->bustard_path();

  my $outfile = $self->make_log_dir( $self->archive_path() ) . q{/} . $self->bsub_output_file();

  my $job_name = q{srf_creation_} . $id_run . q{_} . $timestamp;
  my $replacement = $job_name . q{.%I.%J};
  $job_name .= $arg_refs->{array_string};

  $outfile =~ s/substitute_this/$replacement/xms;

  my $file_name = $self->lsb_jobindex();
  my $qseq_file_string = $self->qseq_file_constructs($file_name);

  if ( ! $qseq_file_string || $qseq_file_string =~ /\A\s*\z/xms ) {
    croak q{No qseq files found which give appropriate match};
  }

  my $srf_file = $self->archive_path() . q{/} . $self->srf_file_constructs;
  $srf_file =~ s/substitute_this/$file_name/xms;

  my $use_bases = $self->corrected_tag_part_use_bases();

  my $job_sub = q{bsub -P pipeline };
  $job_sub .=  ( $self->fs_resource_string( {
    resource_string => q{-R 'select[mem>4000] rusage[mem=4000]' -M4000000}, # whilst 2Gb should be more than enough, requesting 4 ensures that no more than 4 will be launched on any one node, which slows it down
    counter_slots_per_job => 3,
  } ) ) . q{ };
  $job_sub .= qq{$required_job_completion -J $job_name -o $outfile '$script_bin/};
  $job_name .= $self->external_script_names_conf()->{srf_creator_script};
  $job_name .= qq{ $ipar_ints -b -filter-bad-reads};
  if ( $arg_refs->{type_of_lanes} eq q{indexed_lanes} ) {
    $job_name .= q{ -n '%x:%y#%i'};
  } else {
    $job_name .= q{ -n '%x:%y'};
  }
  $job_name .= qq{ -bustard-dir $bustard_dir -use_bases  '$use_bases' -o $srf_file $qseq_file_string'};

  if ($self->verbose()) { $self->log($job_sub); }

  return $job_sub;
}

sub _generate_index_bsub_command {
  my ($self, $arg_refs) = @_;

  my $file_name = $self->lsb_jobindex();
  my $required_job_completion = $arg_refs->{required_job_completion};
  my $timestamp = $self->timestamp();
  my $id_run = $self->id_run();

  my $srf_file = $arg_refs->{srf_file};

  $srf_file ||= $self->archive_path() . q{/} . $self->srf_file_constructs;
  $srf_file =~ s/substitute_this/$file_name/xms;

  my $outfile = $self->make_log_dir( $self->archive_path() ) . q{/} . $self->bsub_output_file();
  my $job_name =  $self->external_script_names_conf()->{srf_indexing_script} . q{_} . $id_run . q{_} . $timestamp;
  my $replacement = $job_name . q{.%I.%J};
  $job_name .= $arg_refs->{array_string};
  $outfile =~ s/substitute_this/$replacement/xms;

  my $job_sub = q{bsub -P pipeline };
  $job_sub .=  ( $self->fs_resource_string( {
    resource_string => q{-R 'select[mem>8000] rusage[mem=8000]' -M8000000}, # upped to 8Gb because HiSeq's need more memory
    counter_slots_per_job => 2,
  } ) ) . q{ };
  $job_sub .= qq{$required_job_completion -J $job_name -o $outfile '} . $self->internal_script_bin() . q{/} . $self->external_script_names_conf()->{srf_indexing_script} . qq{ $srf_file'};

  if ($self->verbose()) { $self->log($job_sub); }

  return $job_sub;
}
no Moose;
__PACKAGE__->meta->make_immutable;
1;
__END__

=head1 NAME

npg_pipeline::archive::file::generation::srf

=head1 VERSION

$LastChangedRevision: 12510 $

=head1 SYNOPSIS

  my $oAfgsrf = npg_pipeline::archive::file::generation::srf->new({
    run_folder => $sRunFolder,
  });

=head1 DESCRIPTION

Object module which knows how to construct and submits the command line to LSF for creating srf files.

=head1 SUBROUTINES/METHODS

=head2 generate - generates the bsub jobs and submits them for creating the srf files, returning an array of job_ids.

  my @job_ids = $oAfgsrf->generate({
    required_job_completion} => q{-w (123 && 321)};
    timestamp                => q{20090709-123456};
    pipeline_bin             => q{/software/pipeline/bin};
    id_run                   => 1234;
    BSUB_ERROR_FILE          => q{substitute_this.err};
    BSUB_OUTPUT_FILE         => q{substitute_this.out};
  });

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Moose

=item Carp

=item English -no_match_vars

=item Readonly

=item npg_pipeline::archive::file::generation

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: ajb $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Andy Brown (ajb@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
