#############
# $Id: fan_out_in.pm 12510 2011-02-01 15:49:31Z ajb $
# Created By: ajb
# Last Maintained By: $Author: ajb $
# Created On: 2009-06-16
# Last Changed On: $Date: 2011-02-01 15:49:31 +0000 (Tue, 01 Feb 2011) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-pipeline/tags/release-24.2/lib/npg_pipeline/pluggable/fan_out_in.pm $

package npg_pipeline::pluggable::fan_out_in;
use Moose;
use Carp;
use English qw{-no_match_vars};
use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$LastChangedRevision: 12510 $ =~ /(\d+)/mxs; $r; };

extends qw{npg_pipeline::pluggable};

Readonly::Scalar our $MAX_LANE => 8;

#######################
## This is an example schema module, which should be removed from production code

sub main {
  my ($self, @args) = @_;
  my $next_function = {
    none => q{lsf_fan_out_in},
    lsf_fan_out_in => q{_test_last},
    last => q{finish},
  }->{$self->completed_function()};
  return $self->$next_function(@args);
}

sub lsf_fan_out_in {
  my ($self, @args) = @_;

  ##############
  # Submit array of jobs to LSF (fan out) and collect job names

  my @jobs = ();

  foreach my $i (1..$MAX_LANE) {
    my $cmd = qq{bsub -q"small" -Ppipeline -J"fan_out_$i" -R "span[hosts=1]" -o /nfs/users/nfs_a/ajb/temp/fan_out_$i.out -e /nfs/users/nfs_a/ajb/temp/fan_out_$i.err '/software/solexa/bin/warehouse_loader 2>&1'};
    my $job = `$cmd`;
    my ($job_id) = $job =~ /(\d+)/xms;
    push @jobs, qq{done($job_id)};
  }

  ##############
  # Submit fan in job to LSF, which will only occur if all jobs have completed
  # This job is responsible for recalling this script (e.g. can be calling this script)

  my $fan_in_cmd = q{bsub};
  my $dependencies = join q{ && }, @jobs;
  $self->log($dependencies);
  $fan_in_cmd .= qq{ -w '$dependencies'};
  $fan_in_cmd .=  q{ -q"small" -Ppipeline -J"calling_fan_in"  -o /nfs/users/nfs_a/ajb/temp/fan_in.out -e /nfs/users/nfs_a/ajb/temp/fan_in.err '/nfs/users/nfs_a/ajb/development/useful_modules/branches/current/scripts/}.$self->script_name().q{ -id_run=}.$self->id_run().q{ -lane=}.$self->lane().q{ -completed_function=lsf_fan_out_in -schema=}.$self->schema();
  if ($self->other_params()) {
    $fan_in_cmd .= q{ -other_params=}.$self->other_params();
  }
  if ($self->alert()) {
    $fan_in_cmd .= q{ -alert=}.$self->alert();
  }
  $fan_in_cmd .= q{'};
  my $job_id = `$fan_in_cmd`;
  $self->log(qq{Fan in job_id = $job_id});
  return 1;
}

no Moose;
__PACKAGE__->meta->make_immutable;
1;
__END__

=head1 NAME

  npg_pipeline::pluggable::fan_out_in

=head1 VERSION

$LastChangedRevision: 12510 $

=head1 SYNOPSIS

This is an example schema module. For usage, see npg_pipeline::pluggable

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2 main

=head2 lsf_fan_out_in - method which tries to submit a range of jobs which can run in parallel, and the collates them together when the next stage requires them all to have completed

For all other methods, see npg_pipeline::pluggable

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item strict

=item warnings

=item Carp

=item English -no_match_vars

=item Readonly

=item base

=item npg_pipeline::pluggable

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: ajb $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Andy Brown (ajb@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
