#############
# $Id: harold.pm 12510 2011-02-01 15:49:31Z ajb $
# Created By: ajb
# Last Maintained By: $Author: ajb $
# Created On: 2009-06-11
# Last Changed On: $Date: 2011-02-01 15:49:31 +0000 (Tue, 01 Feb 2011) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-pipeline/tags/release-24.2/lib/npg_pipeline/pluggable/harold.pm $

package npg_pipeline::pluggable::harold;
use Moose;
use MooseX::InsideOut;
use Carp;
use English qw{-no_match_vars};
use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$LastChangedRevision: 12510 $ =~ /(\d+)/mxs; $r; };
use npg_common::request;
use npg_pipeline::archive::folder::WebCache;

#use SangerPaths qw{trackingnpg};

extends q{npg_pipeline::pluggable};
with q{npg_pipeline::roles::generic::dispatch_tree};
with q{npg_common::roles::run::fs_resource};
with q{npg_pipeline::roles::business::tag_info};
with q{npg_common::roles::run::status};

has q{precreated_function_order} => (isa => q{Str},       is => q{rw});
has q{_function_order}           => (isa => q{ArrayRef},  is => q{ro}, lazy_build => 1);

# reworking needed to take this and $ARCHIVE_RUN out of here!
sub BUILD {
  my ($self) = @_;

  if ( !$self->has_id_run() && !$self->has_run_folder() ) {
    croak qq{\n\n**** No run_folder or id_run provided ****\n\n};
  }

  if (!$self->has_log_file_path()) {
    $self->set_log_file_path( $self->runfolder_path() );
  }

  if (!$self->has_log_file_name()) {
    my $log_name = $PROGRAM_NAME . q{_} . $self->timestamp() . q{.log};
    $log_name =~ s{/}{_}gmxs; # if $PROGRAM_NAME includes a directory path, then in log_name, change / to _
    $log_name =~ s{\A_}{}xms;
    $self->set_log_file_name($log_name);
  };

  if (!$self->precreated_function_order() && !ref$self->function_order()) {
    $self->log(q{You have not declared any function order. Is this correct? Continuing with default.
If you wish to change this, you can either:
  $} . q{Object->precreated_function_order(<archive_run>)
or
  $} . q{Object->function_order([qw(your order of functions here)])});
  }

  # for RTA runs, the intensities directory is generated on the instrument. If it hasn't been
  # already given, then set this up for all analysis
  if ( ! $self->has_intensity_path() ) {
    $self->_set_intensity_path( $self->runfolder_path() . q{/Data/Intensities} );
  }

  return 1;
}

sub _build__function_order {
  my ($self) = @_;

  my $function_order = [];
  if ( ref $self->function_order() eq q{ARRAY} ) {
    $function_order = $self->function_order();
  }

  if ( $self->precreated_function_order() ) {
    $function_order = $self->precreated_function_order() eq q{archive_run} ? $self->function_order_conf()->{post_qc_review}
                    :                                                        []
                    ;
  }

  if ( scalar @{$function_order} < 1 ) {
    $function_order = $self->_default_function_order();
  }

  return $function_order;
}

# redefine this in any subclass if you want a specific default order
sub _default_function_order {
  my ( $self ) = @_;
  return [ qw{finish} ];
}

sub main {
  my ($self, @args) = @_;

  my @functions_to_run_in_order = @{$self->_function_order()};

  my %parallelise_functions     = %{ $self->parallelise() };

  my @job_ids;
  my $all_job_ids = $self->all_job_ids();
  my @hold_parallel_ids;

  ##############
  # run through the selected functions in order selected

  my $add_to_hold;
  foreach my $function (@functions_to_run_in_order) {

    $self->log(q{****** Processing }.$function.q{ ******});
    my $job_reqs = $self->lsf_job_complete_requirements(@job_ids);
    $self->log(qq{$job_reqs for $function});
    unshift @args, $job_reqs;
    ##############
    # if the previous function was parallelisable
    my $parallel_done;
    if ($add_to_hold) {

      my $parallelise_string = q{};
      foreach my $function (sort keys %{$parallelise_functions{$add_to_hold}}) {
        $parallelise_string .= qq{$function }
      }

      ###############
      # check if this function is also part of the same parallelisable group
      if($parallelise_functions{$add_to_hold}->{$function}) {
        ###############
        # if so, then run, and capture the job ids
        $self->log(qq{Able to parallelise $parallelise_string});
        my $job_deps = $args[0];
        my @jids = $self->$function(@args);        # call the function <--------- !!!
        push @hold_parallel_ids, @jids;
        $self->dispatch_tree->append_to_functions({ function => $function,
            job_ids_launched => \@jids,
            job_dependencies => $job_deps,
          });
        $parallel_done++;
      } else {
        $self->log(qq{$function not parallelisable with $parallelise_string\n\tremoving parallelisable status and setting job id requirements for next level});
        $add_to_hold = q{};
        if (scalar @hold_parallel_ids) {
          @job_ids = @hold_parallel_ids;
        }
        push @{$all_job_ids}, @hold_parallel_ids;
        $job_reqs = $self->lsf_job_complete_requirements(@job_ids);
        $self->log(qq{$job_reqs for $function});
        shift@args;
        unshift @args, $job_reqs;
        @hold_parallel_ids = ();
      }
    }
    if (!$parallel_done) {
      my $done;
      foreach my $key (sort keys%parallelise_functions) {
        if($parallelise_functions{$key}->{$function}) {
          $add_to_hold = $key;
          my $parallelise_string = q{};
          foreach my $function (sort keys %{$parallelise_functions{$add_to_hold}}) {
            $parallelise_string .= qq{$function }
          }
          $self->log(qq{Able to parallelise $parallelise_string});
          my $job_deps = $args[0];
          my @jids = $self->$function(@args);        # call the function <--------- !!!
          push @hold_parallel_ids, @jids;
          $self->dispatch_tree->append_to_functions({ function => $function,
            job_ids_launched => \@jids,
            job_dependencies => $job_deps,
          });
          $done++;
          last;
        }
      }
      if (!$done) {
        $self->log(qq{$function is not parallelisable});
        my $job_deps = $args[0];
        my @returned_job_ids = $self->$function(@args);        # call the function <--------- !!!
        if (scalar @returned_job_ids) {
          @job_ids = @returned_job_ids;
        }
        push @{$all_job_ids}, @returned_job_ids;
        $self->dispatch_tree->append_to_functions({ function => $function,
          job_ids_launched => \@returned_job_ids,
          job_dependencies => $job_deps,
        });
      }
    }
  }

  return 1;
}

##############
# qseqs have been coming through without their id_runs in them. This submits jobs to fix that.
sub fix_qseq_run_id {
  my ($self, @args) = @_;
  my @job_ids;
  eval {
#    require npg_pipeline::archive::file::fix_qseqs;
    my $f_qseqs = $self->new_with_cloned_attributes(q{npg_pipeline::archive::file::fix_qseqs});
    my $arg_refs = {
      required_job_completion  => shift @args,
      timestamp                => $self->timestamp(),
      id_run                   => $self->id_run(),
      BSUB_ERROR_FILE          => $self->bsub_error_file(),
      BSUB_OUTPUT_FILE         => $self->bsub_output_file(),
    };
    @job_ids = $f_qseqs->fix_qseqs($arg_refs);
    1;
  } or do {
    croak $EVAL_ERROR;
  };
  return @job_ids;
}

#################
# methods for running analysis pipeline

# all below after bustard has completed

# should be optional
sub adaptor_removal {
  my ($self, @args) = @_;
  my $required_job_completion = shift @args;
  my $job_id;
  return ($job_id);
}

# after adaptor_removal if chosen, otherwise first in line
sub split_by_tag {
  my ($self, @args) = @_;
  my $required_job_completion = shift @args;
  my $job_id;
  return ($job_id);
}

# performed on the control lane and/or 1% spike - requires split by tag
sub create_calibration_table {
  my ($self, @args) = @_;
  my $required_job_completion = shift @args;
  my @job_ids;
  eval {
#    require npg_pipeline::run::calibration;
    my $r_cal = $self->new_with_cloned_attributes(q{npg_pipeline::run::calibration});
    @job_ids = $r_cal->create_table();
  } or do {
    croak $EVAL_ERROR;
  };
  return @job_ids;
}

# once calibration table created from spike control - requires create_calibration_table
sub calibrate_scores {
  my ($self, @args) = @_;
  my $required_job_completion = shift @args;
  my @job_ids;
  eval {
#    require npg_pipeline::run::calibration;
    my $r_cal = $self->new_with_cloned_attributes(q{npg_pipeline::run::calibration});
    @job_ids = $r_cal->create_apply();
  } or do {
    croak $EVAL_ERROR;
  };
  return @job_ids;
}

# should be optional, but after calibrate_scores (and consent_alignment if chosen)
sub kmer_error_correction {
  my ($self, @args) = @_;
  my $required_job_completion = shift @args;
  my $job_id;
  return ($job_id);
}

# should be optional, to outut BAM, but needs to be after create_fastq
sub align_to_ref {
  my ($self, @args) = @_;
  my $required_job_completion = shift @args;
  my $job_id;
  return ($job_id);
}

################
# methods for creating summary links

sub create_summary_link_analysis {
  my ($self, @args) = @_;
  return $self->_create_summary_link(q{full},q{analysis},@args);
}

sub create_summary_link_analysis_prelim {
  my ($self, @args) = @_;
  return $self->_create_summary_link(q{single}, q{analysis},@args);
}

# used in archival step (post qc review)
sub create_summary_link_outgoing {
  my ($self, @args) = @_;
  return $self->_create_summary_link(q{full}, q{outgoing},@args);
}

sub _create_summary_link {
  my ($self, $analysis_type, $folder, @args) = @_;

  # check that this hasn't been expressly turned off
  if ($self->no_summary_link()) {
    $self->log(q{Summary link creation turned off});
    return ();
  }

  my $required_job_completion = shift @args;

  my $rfl = $self->new_with_cloned_attributes(q{npg_pipeline::run::folder::link},{
    analysis_type => $analysis_type,
    folder        => $folder,
  });

  my $arg_refs = {
    required_job_completion => $required_job_completion,
  };

  if (!$required_job_completion) {
    $rfl->make_link($arg_refs);
    return ();
  }

  my $job_id = $rfl->submit_create_link($arg_refs);
  return ($job_id);
}

# used in archival step (post qc review)
sub move_to_outgoing {
  my ($self, @args) = @_;

  # check that this hasn't been expressly turned off
  if ($self->no_folder_moves()) {
    $self->log(q{Folder moving turned off});
    return ();
  }

  my $required_job_completion = shift @args;
#  require npg_pipeline::run::folder::move;
  my $rfm = $self->new_with_cloned_attributes(q{npg_pipeline::run::folder::move},{
    folder => q{outgoing},
  });

  my $arg_refs = {
    required_job_completion => $required_job_completion,
  };

  if (!$required_job_completion) {
    $rfm->move_runfolder($arg_refs);
    $self->create_summary_link_outgoing(); # the latest summary link will need updating
    return ();
  }

  my $job_id = $rfm->submit_move_run_folder($arg_refs);
  return ($job_id);

}

##############
# methods responsible for setting run statuses
sub set_run_status_analysis_in_progress {
  my ($self, @args) = @_;
  return $self->_update_run_status( q{STATUS_ANALYSIS_IN_PROGRESS}, @args );
}

sub set_run_status_secondary_analysis_in_progress {
  my ($self, @args) = @_;
  return $self->_update_run_status( q{STATUS_SECONDARY_ANALYSIS_IN_PROGRESS}, @args );
}

sub set_run_status_analysis_complete {
  my ($self, @args) = @_;
  return $self->_update_run_status( q{STATUS_ANALYSIS_COMPLETE}, @args );
}

sub set_run_status_qc_review_pending {
  my ($self, @args) = @_;
  return $self->_update_run_status( q{STATUS_QC_REVIEW_PENDING}, @args );
}

sub set_run_status_archival_in_progress {
  my ($self, @args) = @_;
  return $self->_update_run_status( q{STATUS_ARCHIVAL_IN_PROGRESS}, @args );
}

sub set_run_status_run_archived {
  my ($self, @args) = @_;
  return $self->_update_run_status( q{STATUS_RUN_ARCHIVED}, @args );
}

sub set_run_status_qc_complete {
  my ($self, @args) = @_;
  return $self->_update_run_status( q{STATUS_QC_COMPLETE}, @args );
}

sub _update_run_status {
  my ($self, $status, @args) = @_;

  $status = $self->run_statuses( { util => $self->run()->util() } )->{ $status };

  # check that this hasn't been expressly turned off
  if ($self->no_status_updates()) {
    $self->log(q{Update status turned off});
    return ();
  }

  local $ENV{npg_common::request->cache_dir_var_name} = q[]; #use of cache can stop updating when we should update RT177301
  my $required_job_completion = shift @args;

  my $rsu = $self->new_with_cloned_attributes(q{npg_pipeline::run::status::update},{
    status_one => $status,
    status_two => $status,
  });
  my $arg_refs = {
    required_job_completion => $required_job_completion,
  };

  if (!$required_job_completion) {
    $rsu->update($arg_refs);
    return ();
  }

  my $job_id = $rsu->submit_update($arg_refs);
  return ($job_id);
}

sub _autoqc_options_from_args {
  my ($self, $arg_refs, @args) = @_;

  if (!defined $arg_refs) {
    $arg_refs = {};
  }
  if (@args) {
    my $options = pop @args;
    if (ref $options eq q[HASH] && exists $options->{autoqc_data} && ref $options->{autoqc_data} eq q[HASH] ) {
      foreach my $key (keys %{$options->{autoqc_data}}) {
	$arg_refs->{$key} = $options->{autoqc_data}->{$key};
      }
    }
  }
  return $arg_refs;
}

################
# single method to actually run qc, which should be given the test from the called method requested
sub _qc_runner {
  my ($self, $qc_to_run, @args) = @_;
  my $required_job_completion = shift @args;

  my $afg = $self->new_with_cloned_attributes(q{npg_pipeline::archive::file::qc},{
    qc_to_run  => $qc_to_run,
  });

  my $arg_refs = {
    required_job_completion => $required_job_completion,
  };

  my @job_ids = $afg->run_qc($self->_autoqc_options_from_args($arg_refs, @args));
  return @job_ids;
}

##############
# spider out to retrieve and cache webservices
# this is not to be submitted as a bsub job
sub spider {
  my ( $self, @args ) = @_;

  my $cache_dir_var_name;
  if ( $self->no_spider() && $self->no_cached_webservice_data() ) {
    $self->log( q{both spidering and using cached webservice data turned off} );
  } else {
    $cache_dir_var_name = npg_common::request->cache_dir_var_name();

    # if the environment variable is already set, then there is no need to spider out
    if ( $ENV{ $cache_dir_var_name } ) {
      $self->log( q[$]. q[ENV{] . $cache_dir_var_name . q[} already set to ] . $ENV{$cache_dir_var_name} . q{, not spidering!} );
      return ();
    }

    # see if the webcache directory is linked in the archive directory, and use this if it is
    # a new object is used, so as to safeguard directory structures that have not yet been created
    my $linked_cache = $self->new_with_cloned_attributes( q{npg_pipeline::archive::folder::WebCache} )->webcache_directory_link();

    # use a provided folder, or a linked_cache, or generate it
    my $cache_dir = $self->has_webservice_cache_dir() ? $self->webservice_cache_dir()
                  : $linked_cache                     ? $linked_cache
                  :                                     $self->runfolder_path() . q{/cache_webservice_analysis_} . $self->timestamp();
                  ;

    # if the cache dir doesn't already exist, create it
    if ( ! ( -d $cache_dir ) ) {
      my $output = qx{mkdir $cache_dir};
      if ( $CHILD_ERROR ) {
        croak q{problem creating cache directory: } . $cache_dir;
      }
    }

    $self->log( q[Setting $] . q[ENV{] . $cache_dir_var_name . q[} to ] . $cache_dir );
    $ENV{$cache_dir_var_name} = $cache_dir; ## no critic (Variables::RequireLocalizedPunctuationVars)

  }
  if ( $self->no_spider() ) {
    $self->log( q{spidering turned off by user} );
  } else {

    my $spider = $self->new_with_cloned_attributes( q{npg_pipeline::spider::AllRunInfo}, {
      save2cache => 1,
    } );
    eval {
      $spider->spider_all();
      1;
    } or do {
      $self->log( q{Somethind went wrong with spidering: } . $EVAL_ERROR );
      croak $EVAL_ERROR;
    };

  }

  # ensure that there is no environment variable set if there is to be no cached services used
  if ( $self->no_cached_webservice_data() ) {
    $self->log( q{using webservice cached responses turned off by user} );
    $ENV{$cache_dir_var_name} = undef; ## no critic (Variables::RequireLocalizedPunctuationVars)
  }

  return ();
}

# check config files have correct data, and fix runfolder,instrument,id_run
# this will not be a job launched on the farm, and so should be taken into account
# i.e. don't expect it to run after something has been submitted, expecting a dependency
sub fix_config_files {
  my ( $self, @args ) = @_;

  if ( ! $self->no_fix_config_files() ) {
    $self->new_with_cloned_attributes( q{npg_pipeline::analysis::FixConfigFiles} )->run();
  }

  return ();
}


# provided builder for parallelise to take the parallelise hash out of a config file,
# if it hasn't already been provided by the user
sub _build_parallelise {
  my ( $self ) = @_;

  return $self->parallelisation_conf();
}

no Moose;
__PACKAGE__->meta->make_immutable;
1;
__END__

=head1 NAME

npg_pipeline::pluggable::harold

=head1 VERSION

$LastChangedRevision: 12510 $

=head1 SYNOPSIS

  my $harold;
  eval {
    $harold = npg_pipeline::pluggable::harold->new({
      schema => q{harold},
      script_name => q{test},
      id_run => 1234,
      run_folder => q{123456_IL2_1234},
    });
  };

You can also pass in an 'alert' parameter, which will email on completion the log and list of job ids to the email given

You can also pass in a path to a GERALD folder 'analysis_path', which will then be used  to work out the locations of files
instead of the best fit GERALD folder it can find (most recent).

=head1 DESCRIPTION

A npg_pluggable::pipeline schema for use in replacing some of the end of the current illumina pipeline process

=head1 SUBROUTINES/METHODS

=head2 BUILD

Called on new construction to ensure that certain parameters are filled/set up

=head2 main

Method to be called to run the pipeline once the pluggable::harold object created.

  eval {
    $pipeline_harold->main();
  } or do {
    # your error handling here
  };

=head2 create_summary_link_analysis_prelim
=head2 create_summary_link_analysis
=head2 create_summary_link_outgoing

functions which change where the summary link will point to

=head2 set_run_status_analysis_complete
=head2 set_run_status_qc_complete
=head2 set_run_status_qc_review_pending
=head2 set_run_status_run_archived
=head2 set_run_status_archival_in_progress
=head2 set_run_status_analysis_in_progress
=head2 set_run_status_secondary_analysis_in_progress

functions to update the run statuses

=head2 adaptor_removal
=head2 align_to_ref
=head2 bustard
=head2 calibrate_scores
=head2 create_calibration_table
=head2 split_nonconsented_sequence
=head2 kmer_error_correction
=head2 split_by_tag
=head2 move_to_outgoing
=head2 fix_config_files

=head2 fix_qseq_run_id
Should hopefully be able to be removed

=head2 spider_webservices
Function which will go out and spider over webservices to cache on the filesystem all data from npg and sequencescape
This should be run up front of any jobs being submitted, and will not submit any jobs to lsf
If the environment variable is already set which is to be set for spidering, it will assume that it has already been
done, so post_qseq can have it as a default to run, but not redo if launched from analyse_RTA

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Moose

=item MooseX::InsideOut

=item Carp

=item English -no_match_vars

=item Readonly

=item base

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: ajb $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Andy Brown (ajb@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
