# $Id: 10-pluggable.t 12510 2011-02-01 15:49:31Z ajb $
use strict;
use warnings;
use Carp;
#use SangerPaths qw{trackingnpg};
use English qw{-no_match_vars};
use Test::More tests => 26;
use Test::Exception;
use lib qw{t};
use t::util;

use_ok('npg_pipeline::pluggable');

my $util = t::util->new();
$util->catch_email();
my $conf_path = $util->conf_path();

$ENV{TEST_BIN} = q{t/bin/};
$ENV{TEST_DIR} = $util->temp_directory();
$ENV{OWNING_GROUP} = q{staff};
$ENV{TEST_API_UTIL} = $util->apiutil();

{
  my $pluggable = npg_pipeline::pluggable->new({
    id_run => 1234,
    schema => q{test},
    analysis_path => q{Data/found/here},
    conf_path => $conf_path,
    domain => q{test},
  });
  isa_ok($pluggable, q{npg_pipeline::pluggable}, q{$pluggable});
  my $finish;
  lives_ok { $finish = $pluggable->finish(); 1; } q{no croak with $pluggable->finish() with no alert};
  is($finish, undef, q{return value of $pluggable->finish() is correct});
  is($pluggable->analysis_path(), q{Data/found/here}, q{$pluggable->analysis_path() populates on new});
  $pluggable->{analysis_path} = q{aaahhhh};
  is($pluggable->analysis_path(), q{Data/found/here}, q{$pluggable->analysis_path() has not altered, even though the key with the same name has});
}

{
  my $pluggable = npg_pipeline::pluggable->new({
    id_run => 1234,
    lane   => 1,
    alert => q{test_user},
    other_params => q{key1=val1:key2=val2},
    conf_path => $conf_path,
    domain => q{test},
  });

  is($pluggable->analysis_path(), q{}, q{$pluggable->analysis_path() set to empty string by a build method within the role it comes from});
  is($pluggable->id_run(), 1234, q{$pluggable->id_run() populated on new});
  is($pluggable->lane(), 1, q{$pluggable->lane() populated on new});
  is($pluggable->script_name(), q{t/10-pluggable.t}, q{$pluggable->script_name() obtained});
  is($pluggable->alert(), q{test_user}, q{$pluggable->alert() populated on new});
  is($pluggable->other_params(), q{key1=val1:key2=val2}, q{$pluggable->other_params() populated on new});

  is($pluggable->main(), 1, q{$pluggable->main() returns 1});
  my $finish;
  lives_ok { $finish = $pluggable->finish(); 1; } q{no croak with $pluggable->finish() with no alert};
  is($finish, undef, q{return value of $pluggable->finish() is correct});
  my $mail = $util->parse_email();
  is($mail->{to}, q{test_user@sanger.ac.uk}.qq{\n}, q{to is correct});
  is($mail->{from}, q{pipeline@test.org}.qq{\n}, q{from is correct});
  is($mail->{subject}, qq{Functional script t/10-pluggable.t schema: npg_pipeline::pluggable\n}, q{subject is correct});
  my $expected_body = qr{Parameters:
id_run:[ ]1234
lane:[ ]1
script_name:[ ]t/10-pluggable.t
alert:[ ]test_user[@]sanger.ac.uk
other_params:[ ]key1=val1:key2=val2

Cheers

New Pipeline Development

};

  like($mail->{body}, $expected_body, q{body is correct});
  is($mail->{precendence}, qq{list\n}, q{precendence is correct});
  is($mail->{cc}, undef, q{cc is correct});
  is($mail->{bcc}, undef, q{bcc is correct});
  is($mail->{content_type}, qq{text/plain\n}, q{content_type is correct});

  my $other_params;
  lives_ok { $other_params = $pluggable->other_params_hash(); } q{no croak $pluggable->other_params_hash()};
  isa_ok($other_params, q{HASH}, q{$pluggable->other_params_hash()});
  my $test_params = {
    key1 => q{val1},
    key2 => q{val2},
  };
  is_deeply($other_params, $test_params, q{$pluggable->_process_other_params() returned the correct data structure});
}

1;