# $Id: 15-pipeline_launcher_scripts.t 12510 2011-02-01 15:49:31Z ajb $
use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use Test::More tests => 26;
use Test::Exception;
use lib qw{t};
use t::util;
use File::Slurp;

my $util = t::util->new();
$util->catch_email();

my $curdir = $util->cwd();
$ENV{TEST_BIN} = $curdir . q{/t/bin/};
$ENV{TEST_DIR} = $util->temp_directory();
my $tmp_dir = $util->temp_directory();

my $conf_path = $curdir . q{/data/config_files};

$ENV{OWNING_GROUP} = q{staff};


my $analysis_runfolder_path = $util->analysis_runfolder_path();
my $outgoing_runfolder_path = $util->outgoing_runfolder_path();
$ENV{TEST_USERAGENT} = $tmp_dir . q{/useragent};

$util->set_rta_staging_analysis_area();
write_file( $ENV{TEST_USERAGENT}, $util->serialized_useragent() );
# This test needs some looking at
#{
#  lives_ok { qx{perl bin/npg_pipeline_analyse_RTA --id_run=1234 --illumina_pipeline=software/solexa/src/GAPipeline --casava_pipeline=software/solexa/src/GAPipeline --verbose --no_control_lane}; } q{ran bin/npg_pipeline_analyse_RTA with id_run for running with recalibrating with no_contro_lane};
#  ok(!$CHILD_ERROR, qq{Return code of $CHILD_ERROR});
#}

# Script failures
{
  lives_ok { qx{perl bin/npg_pipeline_post_qseq --domain test --conf_path $conf_path}; } q{ran bin/npg_pipeline_post_qseq with no id_run};
  ok($CHILD_ERROR, qq{Return code of $CHILD_ERROR as no id_run or runfolder provided});
}
{
  lives_ok { qx{perl bin/npg_pipeline_post_qc_review --domain test --conf_path $conf_path }; } q{ran bin/npg_pipeline_post_qc_review with no id_run};
  ok($CHILD_ERROR, qq{Return code of $CHILD_ERROR as no id_run or runfolder provided});
}
{
  lives_ok { qx{perl bin/npg_pipeline_analyse_RTA --no_fix_config_files --domain test --conf_path $conf_path --fix_broken_files}; } q{ran bin/npg_pipeline_analyse_RTA with no id_run};
  ok($CHILD_ERROR, qq{Return code of $CHILD_ERROR as no id_run or runfolder provided});
}

# Script passes
{
  lives_ok { qx{perl bin/npg_pipeline_analyse_RTA --no_fix_config_files --domain test --conf_path $conf_path --fix_broken_files --id_run=1234 --illumina_pipeline=software/solexa/src/GAPipeline --casava_pipeline=software/solexa/src/GAPipeline --verbose --no_post_qseq --no_eland --no_spider --no_cached_webservice_data}; } q{ran bin/npg_pipeline_analyse_RTA with id_run};
  ok(!$CHILD_ERROR, qq{Return code of $CHILD_ERROR});
}

$util->set_rta_staging_analysis_area();
{
  lives_ok { qx{perl bin/npg_pipeline_analyse_RTA --no_fix_config_files --domain test --conf_path $conf_path --fix_broken_files --id_run=1234 --illumina_pipeline=software/solexa/src/GAPipeline --casava_pipeline=software/solexa/src/GAPipeline --processor_fork_number 6 --post_qseq_executable /post_qseq_test --verbose --no_eland --no_spider --no_cached_webservice_data}; } q{ran bin/npg_pipeline_analyse_RTA with id_run};
  ok(!$CHILD_ERROR, qq{Return code of $CHILD_ERROR});
}

{
  lives_ok { qx{perl bin/npg_pipeline_post_qseq --ref_repository $curdir/t/data/sequence/references --analyse_rta_log_file /staging/IL2/analysis/123456_IL2_1234/npg_pipeline_analyse_RTA_20091028-101635.log --domain test --conf_path $conf_path --id_run=1234 --verbose --no_eland --no_spider --no_cached_webservice_data}; } q{ran bin/npg_pipeline_post_qseq with id_run};
  ok(!$CHILD_ERROR, qq{Return code of $CHILD_ERROR});
}

{
  lives_ok { qx{perl bin/npg_pipeline_post_qc_review --domain test --conf_path $conf_path --id_run=1234 --no_folder_moves --no_status_updates --no_eland --no_spider --no_cached_webservice_data}; } q{ran bin/npg_pipeline_post_qc_review with id_run};
  ok(!$CHILD_ERROR, qq{Return code of $CHILD_ERROR});
}

qx{rm -rf $analysis_runfolder_path/Data/Intensities/Bustard1.5.1_09-07-2009_RTA};
`mkdir -p $tmp_dir/staging/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard_RTA/GERALD_RTA`;
`mkdir -p $tmp_dir/staging/IL2/analysis/123456_IL2_1234/Data/Intensities/BaseCalls/PB_cal`;
`cp t/data/summary_files/after_v7_mp_hack_Summary.xml $tmp_dir/staging/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard_RTA/GERALD_RTA/Summary.xml`;
`cp t/data/summary_files/after_v7_mp_hack_Summary.htm $tmp_dir/staging/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard_RTA/GERALD_RTA/Summary.htm`;
`cp t/data/summary_files/Summary.xsl $tmp_dir/staging/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard_RTA/GERALD_RTA/`;
`cp t/data/runfolder/Data/Intensities/single_end_3cycle_1tile_config.xml $tmp_dir/staging/IL2/analysis/123456_IL2_1234/Data/Intensities/config.xml`;

foreach my $lane ( 1..8 ) {
  foreach my $cycle ( 1..3 ) {
    qx{mkdir -p $tmp_dir/staging/IL2/analysis/123456_IL2_1234/Data/Intensities/BaseCalls/L00$lane/C$cycle.1};
    qx{mkdir -p $tmp_dir/staging/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard_RTA/L00$lane/C$cycle.1};
    qx{mkdir -p $tmp_dir/staging/IL2/analysis/123456_IL2_1234/Data/Intensities/L00$lane/C$cycle.1};
    qx{rm -f $tmp_dir/staging/IL2/analysis/123456_IL2_1234/Data/Intensities/L00$lane/C$cycle.1/s_${lane}_1.cif.ORIGINAL};
    qx{cp t/data/runfolder/Data/Intensities/demo_cif.cif $tmp_dir/staging/IL2/analysis/123456_IL2_1234/Data/Intensities/L00$lane/C$cycle.1/s_${lane}_1.cif};
    qx{touch $tmp_dir/staging/IL2/analysis/123456_IL2_1234/Data/Intensities/L00$lane/C$cycle.1/s_${lane}_1.dif};
    qx{touch $tmp_dir/staging/IL2/analysis/123456_IL2_1234/Data/Intensities/BaseCalls/L00$lane/C$cycle.1/s_${lane}_1.bcl};
    qx{touch $tmp_dir/staging/IL2/analysis/123456_IL2_1234/Data/Intensities/BaseCalls/L00$lane/C$cycle.1/s_${lane}_1.stats};
  }
    qx{touch $tmp_dir/staging/IL2/analysis/123456_IL2_1234/Data/Intensities/BaseCalls/s_${lane}_0001.filter};
}

my $ref_repository = $util->create_references_directory();
# not really happy that this next one is saying no_recalibration, need to work on it
{
  lives_ok { qx{perl $curdir/bin/npg_pipeline_PB_cal --no_fix_config_files --webservice_cache_dir $curdir/t/data --domain test --conf_path $conf_path --fix_broken_files --verbose --tilelayout_rows 60 --lane_count 8 --expected_cycle_count 37 --run_folder 123456_IL2_1234 --array_cpu_limit 64 --processor_fork_number 6  --tilelayout_columns 2 --no_spider --post_qseq_executable /post_qseq_test --id_run 1234 --require_memory_log --runfolder_path $tmp_dir/staging/IL2/analysis/123456_IL2_1234 --recalibrated_path $tmp_dir/staging/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard_RTA/PB_cal --pb_cal_path $tmp_dir/staging/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard_RTA/PB_cal --no_recalibration}; } q{ran bin/npg_pipeline_PB_cal with id_run};
  ok( !$CHILD_ERROR, qq{Return code of $CHILD_ERROR} );
}

{
  lives_ok { qx{perl bin/npg_pipeline_analyse_RTA --no_fix_config_files --domain test --conf_path $conf_path --fix_broken_files --no_bsub --id_run=1234 --illumina_pipeline=software/solexa/src/GAPipeline --casava_pipeline=software/solexa/src/GAPipeline --verbose --no_control_lane --no_eland --no_spider --no_cached_webservice_data --intensity_path $tmp_dir/staging/IL2/analysis/123456_IL2_1234/Data/Intensities --bustard_path $tmp_dir/staging/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard_RTA}; } q{ran bin/npg_pipeline_analyse_RTA with id_run};
  ok(!$CHILD_ERROR, qq{Return code of $CHILD_ERROR});
}
note qq{-----------------\nuser phasing, but no user prephasing};
# This one should have a child error, as no prephasing given on the command line
{
  lives_ok { qx{perl bin/npg_pipeline_analyse_RTA --no_fix_config_files --phasing 0.0041 --matrix /a/path/to/some.matrix --domain test --conf_path $conf_path --fix_broken_files --no_bsub --id_run=1234 --illumina_pipeline=software/solexa/src/GAPipeline --casava_pipeline=software/solexa/src/GAPipeline --verbose --no_control_lane --no_eland --no_spider --no_cached_webservice_data --intensity_path $tmp_dir/staging/IL2/analysis/123456_IL2_1234/Data/Intensities --bustard_path $tmp_dir/staging/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard_RTA}; } q{ran bin/npg_pipeline_analyse_RTA with id_run};
  ok($CHILD_ERROR, qq{Return code of $CHILD_ERROR});
}
note qq{-----------------\nuser phasing and prephasing};

{
  lives_ok { qx{perl bin/npg_pipeline_analyse_RTA --no_fix_config_files --phasing 0.0041 --prephasing 0.0037 --matrix /a/path/to/some.matrix --domain test --conf_path $conf_path --fix_broken_files --no_bsub --id_run=1234 --illumina_pipeline=software/solexa/src/GAPipeline --casava_pipeline=software/solexa/src/GAPipeline --verbose --no_control_lane --no_eland --no_spider --no_cached_webservice_data --intensity_path $tmp_dir/staging/IL2/analysis/123456_IL2_1234/Data/Intensities --bustard_path $tmp_dir/staging/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard_RTA}; } q{ran bin/npg_pipeline_analyse_RTA with id_run};
  ok(!$CHILD_ERROR, qq{Return code of $CHILD_ERROR});
}

{
  lives_ok { qx{perl bin/npg_pipeline_post_qseq --ref_repository $curdir/t/data/sequence/references --pb_cal_log_file /staging/IL2/analysis/123456_IL2_1234/npg_pipeline_pb_cal_20091028-101635.log --domain test --conf_path $conf_path --no_srf_generation --lsf_queue test --basecall_path $tmp_dir/staging/IL2/analysis/123456_IL2_1234/Data/Intensities/BaseCalls --intensity_path $tmp_dir/staging/IL2/analysis/123456_IL2_1234/Data/Intensities --bustard_path $tmp_dir/staging/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard_RTA --use_bases Y37 --runfolder_path $tmp_dir/staging/IL2/analysis/123456_IL2_1234 --recalibrated_path $tmp_dir/staging/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard_RTA/GERALD_RTA --run_folder 123456_IL2_1234 --data_path $tmp_dir/staging/IL2/analysis/123456_IL2_1234/Data --id_run 1234 --fs_resource nfs_12 --no_status_updates --no_summary_link --verbose --no_eland --no_spider --no_cached_webservice_data}; } q{ran bin/npg_pipeline_post_qseq with id_run};
  ok(!$CHILD_ERROR, qq{Return code of $CHILD_ERROR});
`perl bin/npg_pipeline_post_qseq --ref_repository $curdir/t/data/sequence/references --pb_cal_log_file /staging/IL2/analysis/123456_IL2_1234/npg_pipeline_pb_cal_20091028-101635.log --domain test --conf_path $conf_path --no_srf_generation --lsf_queue test --basecall_path $tmp_dir/staging/IL2/analysis/123456_IL2_1234/Data/Intensities/BaseCalls --intensity_path $tmp_dir/staging/IL2/analysis/123456_IL2_1234/Data/Intensities --bustard_path $tmp_dir/staging/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard_RTA --use_bases Y37 --runfolder_path $tmp_dir/staging/IL2/analysis/123456_IL2_1234 --recalibrated_path $tmp_dir/staging/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard_RTA/GERALD_RTA --run_folder 123456_IL2_1234 --data_path $tmp_dir/staging/IL2/analysis/123456_IL2_1234/Data --id_run 1234 --fs_resource nfs_12 --no_status_updates --no_summary_link --verbose --no_eland --no_spider --no_cached_webservice_data`;
}

{
  lives_ok { qx{perl bin/npg_pipeline_post_qseq --ref_repository $curdir/t/data/sequence/references --domain test --conf_path $conf_path --no_bsub --no_srf_generation --no_bam_generation --tag_file t/barcodes/sanger.tags --lsf_queue test --basecall_path $tmp_dir/staging/IL2/analysis/123456_IL2_1234/Data/Intensities/BaseCalls --intensity_path $tmp_dir/staging/IL2/analysis/123456_IL2_1234/Data/Intensities --bustard_path $tmp_dir/staging/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard_RTA --use_bases Y37,I6nn --runfolder_path $tmp_dir/staging/IL2/analysis/123456_IL2_1234 --recalibrated_path $tmp_dir/staging/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard_RTA/GERALD_RTA --run_folder 123456_IL2_1234 --data_path $tmp_dir/staging/IL2/analysis/123456_IL2_1234/Data --id_run 1234 --fs_resource nfs_12 --no_status_updates --no_summary_link --verbose --no_eland --no_spider --no_cached_webservice_data}; } q{ran bin/npg_pipeline_post_qseq with id_run};
  ok(!$CHILD_ERROR, qq{Return code of $CHILD_ERROR});
}

`rm -f *out; rm -f Latest_Summary`;
1;