# $Id: 20-archive-mpsa_migration_helper.t 12510 2011-02-01 15:49:31Z ajb $
use strict;
use warnings;
use Test::More;
use File::Temp qw/ tempdir /;
use Test::Exception;
use lib q{t};
use t::util;

if ( ! $ENV{MIGRATION_HELPER} ) {
  my $msg = 'Migration Helper tests switched off.';
  plan( skip_all => $msg );
}

plan( tests => 15 );

my $util = t::util->new();
my $archive = $ENV{TEST_DIR} = $util->temp_directory();
my $paths = join q[ ], q[--archive_path], $archive,
                       q[--bustard_path], $archive,
                       q[--runfolder_path], $archive,
                       q[--recalibrated_path], $archive,
                       q[--conf_path data/config_files],
                       q[--nois_indexed];

use_ok( q{npg_pipeline::archive::mpsa_to_irods_migration_helper} );

sub populate_archive_directory_from_mpsa {
  my $p = shift;
  $p->launch_bsub_job;
  return;
}

sub qc_db_delete_autoqc {
  my $p = shift;
  $p->launch_bsub_job;
  return;
}

sub run_post_qc_review {
  my $p = shift;
  $p->launch_bsub_job;
  return;
}

sub flag_mpsa_migration_finished {
  my $p = shift;
  $p->launch_bsub_job;
  return;
}


{
  my $p;
  lives_ok {
    $p = npg_pipeline::archive::mpsa_to_irods_migration_helper->new( {
      runfolder_path => $archive,
      archive_path   => $archive,
      bustard_path   => $archive,
      conf_path      => q{data/config_files},
      domain => q{test},
      is_indexed     => 0,
      id_run         => 222,
      lanes          => [1, 3, 5],
      no_bsub => 1,
    });
  } q{create object ok};
  isa_ok( $p, q{npg_pipeline::archive::mpsa_to_irods_migration_helper});

  throws_ok {$p->bsub_command('No script for function my_function')} qr// , 'error when no script defined for a function';

  my $timestamp = $p->timestamp;

  my $expected_command = q[bsub -q srpipeline -J populate_archive_directory_from_mpsa_222_135 -o ]
 . $archive . q[/log/populate_archive_directory_from_mpsa_222_135_] .$timestamp . q[_%J.out -R 'rusage[nfs_12=1]' 'fuse2staging --id_run 222] 
 . q[ --lanes 1 --lanes 3 --lanes 5 ] . $paths . q['];;

  is($p->bsub_command('populate_archive_directory_from_mpsa'), $expected_command, 'bsub command generated correctly');
 
  my $bsub = $p->bsub_command('populate_archive_directory_from_mpsa', {required_job_completion => q[-w 'done(4567)'],}); 
  $expected_command = q[bsub -q srpipeline -J populate_archive_directory_from_mpsa_222_135 -o ]
 . $archive . q[/log/populate_archive_directory_from_mpsa_222_135_] .$timestamp . q[_%J.out -R 'rusage[nfs_12=1]' -w 'done(4567)' 'fuse2staging --id_run 222] 
 . q[ --lanes 1 --lanes 3 --lanes 5 ] . $paths . q['];
  is($bsub, $expected_command, 'bsub command with dependencies generated correctly');

  lives_ok { qc_db_delete_autoqc($p) } 'no error when launching a job';
  lives_ok { populate_archive_directory_from_mpsa($p) } 'no error when launching a job';
  lives_ok { run_post_qc_review($p) } 'no error when launching a job';
  lives_ok { flag_mpsa_migration_finished($p) } 'no error when launching a job';
}

{
  my $p;
  lives_ok {
    $p = npg_pipeline::archive::mpsa_to_irods_migration_helper->new( {
      runfolder_path => $archive,
      archive_path   => $archive,
      bustard_path   => $archive,
      conf_path      => q{data/config_files},
      domain => q{test},
      is_indexed     => 0,
      id_run         => 222,
      no_bsub => 1,
    } );
  } q{create object ok};

  throws_ok {populate_archive_directory_from_mpsa($p)} qr/Pipeline function populate_archive_directory_from_mpsa: --lanes option should be set explicitly/, 'error when launching a job without lanes array';
}

{
  my $p;
  lives_ok {
    $p = npg_pipeline::archive::mpsa_to_irods_migration_helper->new( {
      runfolder_path => $archive,
      archive_path   => $archive,
      bustard_path   => $archive,
      conf_path      => q{data/config_files},
      domain => q{test},
      is_indexed     => 0,
      id_run         => 222,
      lanes          => [],
      no_bsub => 1,
    } );
  } q{create object ok};

  throws_ok {main::populate_archive_directory_from_mpsa($p)} qr/Pipeline function populate_archive_directory_from_mpsa: --lanes option should be set explicitly/, 'error when launching a job with an empty lanes array';

  throws_ok {qc_db_delete_autoqc($p)} qr/Pipeline function qc_db_delete_autoqc: --lanes option should be set explicitly/, 'error when launching a job with an empty lanes array';
}

1;