# $Id: 20-archive_file-split_nonconsented.t 12510 2011-02-01 15:49:31Z ajb $
use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use Test::More tests => 17;
use Test::Exception;
use lib qw{t};
use t::util;

use_ok('npg_pipeline::archive::file::split_nonconsented');

my $util = t::util->new();
my $conf_path = $util->conf_path();

$ENV{TEST_BIN} = q{t/bin/};
$ENV{TEST_DIR} = $util->temp_directory();

my $analysis_runfolder_path = $util->analysis_runfolder_path();

{

  my $split_nc = npg_pipeline::archive::file::split_nonconsented->new({
    run_folder => q{123456_IL2_1234},
    timestamp => q{20090709-123456},
    verbose => 1,
    conf_path => $conf_path,
    domain => q{test},
  });
  isa_ok($split_nc, q{npg_pipeline::archive::file::split_nonconsented}, q{$split_nc});

  my $arg_refs = {
    required_job_completion => q{-w'done(123) && done(321)'},
    array_string => q{[1-3,5,7-8]},
  };

  $util->set_rta_staging_analysis_area();

  $split_nc->_populate_directory_paths();

  my $bsub_command = $util->drop_temp_part_from_paths( $split_nc->_generate_bsub_command( $split_nc->lsb_jobindex(), $arg_refs ) );
  my $expected_command = q{bsub -q test -w'done(123) && done(321)' -J split_nonconsented_1234_20090709-123456[1-3,5,7-8] -R 'select[mem>8000] rusage[mem=8000,nfs_12=4] span[hosts=1]' -M8000000 -n4 -o /staging/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA/GERALD_09-07-2009_RTA/archive/log/split_nonconsented_1234_20090709-123456.%I.%J.out -E 't/bin/software/solexa/bin/npg_pipeline_preexec_references' 't/bin/software/solexa/bin/fastq_srf_split -srf /staging/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA/GERALD_09-07-2009_RTA/archive/1234_`echo $LSB_JOBINDEX`.srf -fastq1 /staging/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA/GERALD_09-07-2009_RTA/archive/1234_`echo $LSB_JOBINDEX`.fastq -fork_alignment -id_run 1234 -position `echo $LSB_JOBINDEX` -output_dir /staging/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA/GERALD_09-07-2009_RTA/archive'};
  is ( $bsub_command, $expected_command, q{$split_nc->_generate_bsub_command( $split_nc->lsb_jobindex(), $arg_refs ) result ok} );

  $arg_refs->{paired}++;
  $bsub_command = $util->drop_temp_part_from_paths( $split_nc->_generate_bsub_command( $split_nc->lsb_jobindex(), $arg_refs ) );
  $expected_command = q{bsub -q test -w'done(123) && done(321)' -J split_nonconsented_1234_20090709-123456[1-3,5,7-8] -R 'select[mem>8000] rusage[mem=8000,nfs_12=4] span[hosts=1]' -M8000000 -n4 -o /staging/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA/GERALD_09-07-2009_RTA/archive/log/split_nonconsented_1234_20090709-123456.%I.%J.out -E 't/bin/software/solexa/bin/npg_pipeline_preexec_references' 't/bin/software/solexa/bin/fastq_srf_split -srf /staging/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA/GERALD_09-07-2009_RTA/archive/1234_`echo $LSB_JOBINDEX`.srf -fastq1 /staging/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA/GERALD_09-07-2009_RTA/archive/1234_`echo $LSB_JOBINDEX`_1.fastq -fastq2 /staging/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA/GERALD_09-07-2009_RTA/archive/1234_`echo $LSB_JOBINDEX`_2.fastq -fork_alignment -id_run 1234 -position `echo $LSB_JOBINDEX` -output_dir /staging/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.5.1_09-07-2009_RTA/GERALD_09-07-2009_RTA/archive'};
  is( $bsub_command, $expected_command, q{$split_nc->_generate_bsub_command( $split_nc->lsb_jobindex(), $arg_refs ) paired result ok} );

}
{

  my $split_nc = npg_pipeline::archive::file::split_nonconsented->new({
    run_folder => q{123456_IL2_1234},
    timestamp => q{20090709-123456},
    verbose => 1,
    conf_path => $conf_path,
    domain => q{test},
  });
  isa_ok($split_nc, q{npg_pipeline::archive::file::split_nonconsented}, q{$split_nc});

  my $arg_refs = {
    required_job_completion => q{-w'done(123) && done(321)'},
  };

  $util->set_rta_staging_analysis_area();

  my $cache = $split_nc->_lane_splitting_cache();

  my @jids;
  foreach my $lane (1..8) {
    $cache->{$lane} = q{true};
  }
  lives_ok { @jids = $split_nc->split_run_lanes($arg_refs); } q{no croak running split};

  is(scalar@jids, 2, q{2 job ids returned});
}

{
  $ENV{TEST_BIN} = undef;
  $ENV{TEST_RUN_OBJECT} = npg::api::run->new({
    util => $util->apiutil(),
    id_run => 1234,
  });

  my $split_nc = npg_pipeline::archive::file::split_nonconsented->new({
    run_folder => q{123456_IL2_1234},
    timestamp => q{20090709-123457},
    verbose => 1,
    conf_path => $conf_path,
    domain => q{test},
  });
  $split_nc->_populate_directory_paths();

  is($split_nc->_does_lane_need_splitting(1), q{false}, q{lane 1 does not contain non-consented human});
  is($split_nc->_does_lane_need_splitting(4), q{false}, q{lane 4 is a control lane});
  is($split_nc->_does_lane_need_splitting(7), q{true},  q{lane 7 does contain non-consented human});

  $ENV{TEST_BIN} = q{t/bin/};
  my $arg_refs = {
    required_job_completion => q{-w'done(123) && done(321)'},
  };
  my @jids;
  eval { @jids = $split_nc->split_run_lanes($arg_refs); };
  is(scalar@jids, 2, q{only one lane needed splitting out non-consented human});
}

{

  my $split_nc = npg_pipeline::archive::file::split_nonconsented->new({
    run_folder => q{123456_IL2_1234},
    timestamp => q{20090709-123456},
    lane => 4,
    verbose => 1,
    conf_path => $conf_path,
    domain => q{test},
  });
  isa_ok($split_nc, q{npg_pipeline::archive::file::split_nonconsented}, q{$split_nc});

  my $arg_refs = {};
  $arg_refs->{required_job_completion}  = q{-w'done(123) && done(321)'};

  $util->set_rta_staging_analysis_area();

  $split_nc->_populate_directory_paths();

  my @jids;
  foreach my $lane (1..8) {
    $split_nc->_lane_splitting_cache()->{$lane} = q{true};
  }
  eval { @jids = $split_nc->split_run_lanes($arg_refs); };
  is($EVAL_ERROR, q{}, q{no croak running split});

  is(scalar@jids, 2, q{2 job ids returned});
}

{

  my $split_nc = npg_pipeline::archive::file::split_nonconsented->new({
    run_folder => q{123456_IL2_1234},
    timestamp => q{20090709-123456},
    lane => 4,
    verbose => 1,
    conf_path => $conf_path,
    domain => q{test},
  });
  isa_ok($split_nc, q{npg_pipeline::archive::file::split_nonconsented}, q{$split_nc});

  my $arg_refs = {};
  $arg_refs->{required_job_completion}  = q{-w'done(123) && done(321)'};
  $arg_refs->{multiplexed}              = 4;

  $util->set_rta_staging_analysis_area();

  $split_nc->_populate_directory_paths();

  my @jids;
  foreach my $lane (1..8) {
    $split_nc->_lane_splitting_cache()->{$lane} = q{true};
  }
  eval { @jids = $split_nc->split_run_lanes($arg_refs); };
  is($EVAL_ERROR, q{}, q{no croak running split when there is a multiplexed run});

  is(scalar@jids, 2, q{2 job ids returned});
}


1;
