# $Id: 20-archive_qc.t 12510 2011-02-01 15:49:31Z ajb $
use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use Test::More tests => 26;
use lib qw{t lib};
use t::util;

use_ok('npg_pipeline::archive::file::qc');

my $util = t::util->new();
my $conf_path = $util->conf_path();

$ENV{TEST_BIN} = q{t/bin/};
$ENV{TEST_DIR} = $util->temp_directory();

my $run_folder = q{123456_IL2_1234};
{
  my $aqc;
  eval {
    $aqc = npg_pipeline::archive::file::qc->new({
      run_folder => $run_folder,
      conf_path => $conf_path,
      domain => q{test},
    });
  };
  like($EVAL_ERROR, qr/Attribute[ ][(]qc_to_run[)][ ]is[ ]required/, q{croak on new as no qc_to_run provided});
}
{
  my $aqc;
  my $arg_refs = {};
  $arg_refs->{required_job_completion}  = q{-w'done(123) && done(321)'};
  eval {
    $aqc = npg_pipeline::archive::file::qc->new({
      run_folder => $run_folder,
      qc_to_run => q{qX_yield},
      timestamp => q{20090709-123456},
      conf_path => $conf_path,
      domain => q{test},
#      verbose => 1,
    });
  };
  is($EVAL_ERROR, q{}, q{no croak on new, as required params provided});

  $util->create_analysis({qc_dir => 1});

  my @jids;
  eval { @jids = $aqc->run_qc($arg_refs); };
  is($EVAL_ERROR, q{}, q{no croak $aqc->run_qc()});
  is(scalar@jids, 1, q{1 job id returned});

  my $bsub_command = $util->drop_temp_part_from_paths( $aqc->_generate_bsub_command($arg_refs) );
  my $expected_command = q{bsub -q test -R 'rusage[nfs_12=1]' -w'done(123) && done(321)' -J 'qc_qX_yield_1234_20090709-123456[1-8]%64' -o /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/qc/log/qc_qX_yield_1234_20090709-123456.%I.%J.out 't/bin/software/solexa/bin/qc --check=qX_yield --archive_path=/staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive --id_run=1234 --position=`echo $LSB_JOBINDEX`'};
  is( $bsub_command, $expected_command, q{generated bsub command is correct});
}

{
  my $aqc;
  my $arg_refs = {};
  $arg_refs->{required_job_completion}  = q{-w'done(123) && done(321)'};

  eval {
    $aqc = npg_pipeline::archive::file::qc->new({
      run_folder => $run_folder,
      qc_to_run => q{qX_yield},
      verbose => 1,
      timestamp => q{20090709-123456},
      lane => 4,
      conf_path => $conf_path,
      domain => q{test},
    });
  };
  is($EVAL_ERROR, q{}, q{no croak on new, as required params provided});

  $util->create_analysis({qc_dir => 1});

  my @jids;
  eval { @jids = $aqc->run_qc($arg_refs); };
  is($EVAL_ERROR, q{}, q{no croak $aqc->run_qc()});
  is(scalar@jids, 1, q{1 job id returned});

  my $bsub_command = $util->drop_temp_part_from_paths( $aqc->_generate_bsub_command($arg_refs) );
  my $expected_command = q{bsub -q test -R 'rusage[nfs_12=1]' -w'done(123) && done(321)' -J 'qc_qX_yield_1234_20090709-123456[4]%64' -o /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/qc/log/qc_qX_yield_1234_20090709-123456.%I.%J.out 't/bin/software/solexa/bin/qc --check=qX_yield --archive_path=/staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive --id_run=1234 --position=`echo $LSB_JOBINDEX`'};
  is( $bsub_command, $expected_command, q{generated bsub command is correct});
}

$ENV{TEST_RUN_OBJECT} = npg::api::run->new({
  util => $util->apiutil(),
  id_run => 1234,
});

{

  my $args = {};
  $args->{qc_dir} = [7];

  $util->create_multiplex_analysis($args);
  $util->copy_tag_file(7);

  my $aqc;
  my $arg_refs = {};
  $arg_refs->{required_job_completion}  = q{-w'done(123) && done(321)'};
  $arg_refs->{lane} = 7;

  eval {
    $aqc = npg_pipeline::archive::file::qc->new({
      run_folder => $run_folder,
      qc_to_run => q{qX_yield},
      verbose => 1,
      timestamp => q{20090709-123456},
      lane => 7,
      conf_path => $conf_path,
      domain => q{test},
    });
  };
  is($EVAL_ERROR, q{}, q{no croak on new for a demultiplexed, as required params provided});

  my @jids;
  eval { @jids = $aqc->run_qc($arg_refs); };
  is($EVAL_ERROR, q{}, q{no croak $aqc->run_qc()});
  is(scalar@jids, 2, q{1 job id returned});

  $util->remove_staging;
}

{

  my $args = {};
  $args->{qc_dir} = [7];

  $util->create_multiplex_analysis($args);
  $util->copy_tag_file(7);

  my $aqc;
  my $arg_refs = {};
  $arg_refs->{required_job_completion}  = q{-w'done(123) && done(321)'};
  $arg_refs->{lane} = 7;

  eval {
    $aqc = npg_pipeline::archive::file::qc->new({
      run_folder => $run_folder,
      qc_to_run => q{insert_size},
      verbose => 1,
      timestamp => q{20090709-123456},
      lane => 7,
      conf_path => $conf_path,
      domain => q{test},
    });
  };
  is($EVAL_ERROR, q{}, q{no croak on new for a demultiplexed, as required params provided});

  my @jids;
  eval { @jids = $aqc->run_qc($arg_refs); };
  is($EVAL_ERROR, q{}, q{no croak $aqc->run_qc()});
  is(scalar     @jids, 2, q{2 job ids returned});

  eval {
    $aqc = npg_pipeline::archive::file::qc->new({
      run_folder => $run_folder,
      qc_to_run => q{cache},
      verbose => 1,
      timestamp => q{20090709-123456},
      lane => 7,
      conf_path => $conf_path,
      domain => q{test},
    });
  };
  is($EVAL_ERROR, q{}, q{no croak on new for a demultiplexed, as required params provided});

  @jids = undef;
  eval { @jids = $aqc->run_qc($arg_refs); };
  is($EVAL_ERROR, q{}, q{no croak $aqc->run_qc()});
  is(scalar @jids, 2, q{2 job ids returned});

  my $cmd = q{bsub -q test -R 'rusage[nfs_12=1]' -w'done(123) && done(321)' -J 'qc_cache_1234_20090709-123456[7]%64' -o /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/qc/log/qc_cache_1234_20090709-123456.%I.%J.out -E 't/bin/software/solexa/bin/npg_pipeline_preexec_references' 'perl -e "use lib qw[/software/solexa/lib/perl5]; use npg_common::extractor::fastq qw[generate_cache]; generate_cache( q[/staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive], [ qw{ /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/1234_`echo $LSB_JOBINDEX`_1.fastq /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/1234_`echo $LSB_JOBINDEX`_2.fastq /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/1234_`echo $LSB_JOBINDEX`_t.fastq } ] );"'};
  is( $util->drop_temp_part_from_paths( $aqc->_generate_bsub_command( $arg_refs ) ), $cmd, q{correct lane based cache command} );

  $cmd = q{bsub -q test -R 'rusage[nfs_12=1]' -w'done(123) && done(321)' -J 'qc_cache_1234_20090709-123456[7000,7000,7002,7002,7012,7012]%64' -o /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/qc/log/qc_cache_1234_20090709-123456.%I.%J.out -E 't/bin/software/solexa/bin/npg_pipeline_preexec_references' 'perl -e "use lib qw[/software/solexa/lib/perl5]; use npg_common::extractor::fastq qw[generate_cache]; generate_cache( q[/staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/lane`echo $LSB_JOBINDEX/1000 | bc`], [ qw{ /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/lane`echo $LSB_JOBINDEX/1000 | bc`/1234_`echo $LSB_JOBINDEX/1000 | bc`_1#`echo $LSB_JOBINDEX%1000 | bc`.fastq /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/lane`echo $LSB_JOBINDEX/1000 | bc`/1234_`echo $LSB_JOBINDEX/1000 | bc`_2#`echo $LSB_JOBINDEX%1000 | bc`.fastq } ] );"'};

  is( $util->drop_temp_part_from_paths( $aqc->_generate_bsub_command( $arg_refs, 1 ) ), $cmd, q{correct demultiplex based cache command} );


  $arg_refs->{spiked_phix_sequence} = q[--species PhiX --sequence_type phix];
  $arg_refs->{lane} = 2;

  eval {
    $aqc = npg_pipeline::archive::file::qc->new({
      run_folder => $run_folder,
      qc_to_run => q{sequence_error},
      verbose => 1,
      timestamp => q{20090709-123456},
      lane => 2,
      conf_path => $conf_path,
      domain => q{test},
    });
  };

  @jids = undef;
  eval { @jids = $aqc->run_qc($arg_refs); };
  is($EVAL_ERROR, q{}, q{no croak $aqc->run_qc()});
  is(scalar @jids, 1, q{1 job id returned});

  $cmd = q{bsub -q test -R 'select[type==X86_64,mem>6000] rusage[mem=6000,nfs_12=1]' -M6000000 -w'done(123) && done(321)' -J 'qc_sequence_error_spiked_phix_1234_20090709-123456[2]%64' -o /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/qc/log/qc_sequence_error_spiked_phix_1234_20090709-123456.%I.%J.out -E 't/bin/software/solexa/bin/npg_pipeline_preexec_references' 't/bin/software/solexa/bin/qc --check=sequence_error --archive_path=/staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive --id_run=1234 --position=`echo $LSB_JOBINDEX` --species PhiX --sequence_type phix'};

  is( $util->drop_temp_part_from_paths($aqc->_generate_bsub_command( $arg_refs )), $cmd, q{correct lane sequence_error4spiked_phix command} );

  $arg_refs->{lane} = 7;

  eval {
    $aqc = npg_pipeline::archive::file::qc->new({
      run_folder => $run_folder,
      qc_to_run => q{sequence_error},
      verbose => 1,
      timestamp => q{20090709-123456},
      lane => 7,
      conf_path => $conf_path,
      domain => q{test},
    });
  };

  @jids = undef;
  eval { @jids = $aqc->run_qc($arg_refs); };
  is($EVAL_ERROR, q{}, q{no croak $aqc->run_qc()});
  is(scalar @jids, 0, q{0 job id returned});
  
  $util->remove_staging;
}

1;