# $Id: 25-analysis-create_psuedo_qseq_custom.t 12510 2011-02-01 15:49:31Z ajb $
use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use Test::More tests => 20;
use Test::Exception;
use lib qw{t};
use t::util;

BEGIN {
  use_ok(q{npg_pipeline::analysis::create_psuedo_qseq_custom});
}

my $util = t::util->new();
my $conf_path = $util->conf_path();

my $temp_dir = $util->temp_directory();

my $runfolder_path = $util->analysis_runfolder_path();
my $bustard_home   = qq{$runfolder_path/Data/Intensities};
my $bustard_rta    = qq{$bustard_home/Bustard_RTA};
my $gerald_rta     = qq{$bustard_rta/GERALD_RTA};

sub set_staging_analysis_area {
  my ($indexed) = @_;

  `rm -rf $temp_dir/staging`;
  `mkdir -p $runfolder_path/Config`;
  `mkdir -p $gerald_rta`;
  `touch $bustard_rta/s_1_1_001_qseq.txt`;
  `touch $bustard_rta/s_1_2_001_qseq.txt`;
  `touch $bustard_rta/s_2_1_001_qseq.txt`;
  `touch $bustard_rta/s_2_2_001_qseq.txt`;
  `touch $gerald_rta/s_1_1_001_qseq_custom.txt`;
  `touch $gerald_rta/s_1_2_001_qseq_custom.txt`;

  if ($indexed) {
    `cp t/data/Recipes/Recipe_GA2-PEM_MP_2x76Cycle+8_v7.7.xml $runfolder_path/`;
    `cp t/data/mock_qseqs/1234/s_8_2_0001_qseq.txt $bustard_rta/s_1_t_001_qseq.txt`;
    `touch $bustard_rta/s_2_t_001_qseq.txt`;
  } else {
    `cp t/data/Recipes/Recipe_GA2_37Cycle_PE_v6.1.xml $runfolder_path/`;
  }

  `cp t/data/Recipes/TileLayout.xml $runfolder_path/Config/`;

  return 1;
}

sub strip_preset_custom_qseqs {
  `rm -f $gerald_rta/s_1_1_001_qseq_custom.txt`;
  `rm -f $gerald_rta/s_1_2_001_qseq_custom.txt`;
  return 1;
}

{
  set_staging_analysis_area();
  my $object;
  lives_ok { $object = npg_pipeline::analysis::create_psuedo_qseq_custom->new({
    id_run => 1234,
    run_folder => q{123456_Il2_1234},
    runfolder_path => $runfolder_path,
    bustard_path => $bustard_rta,
    recalibrated_path => $gerald_rta,
    verbose => 1,
    conf_path => $conf_path,
    domain => q{test},
  }); } q{object creation ok};
  isa_ok($object, q{npg_pipeline::analysis::create_psuedo_qseq_custom}, q{$object});

  ok(!$object->control(), q{no control found});

  my $all_qseq_custom_files = $object->qseq_custom_files();
  isa_ok($all_qseq_custom_files, q{HASH}, q{$object->_all_qseq_custom_files()});
  my $hash = {
    q{s_1_1_001_qseq} => 1,
    q{s_1_2_001_qseq} => 1,
  };
  is_deeply($all_qseq_custom_files, $hash, q{returned hash is correct});

  my @all_qseq_files = $object->_all_qseq_files();
  my $array = [qw{s_1_1_001_qseq s_1_2_001_qseq s_2_1_001_qseq s_2_2_001_qseq}];
  is_deeply(\@all_qseq_files, $array, q{return array is correct});

  lives_ok { $object->run(); } q{run is ok - returns quickly since there are qseq_custom files found};
  $object->clear_qseq_custom_files();

  strip_preset_custom_qseqs();

  lives_ok { $object->run(); } q{run is ok};
  $hash->{s_2_1_001_qseq} = 1;
  $hash->{s_2_2_001_qseq} = 1;
  $object->clear_qseq_custom_files();
  $all_qseq_custom_files = $object->qseq_custom_files();
  is_deeply($all_qseq_custom_files, $hash, q{qseq_custom_files are all there});
}
{
  set_staging_analysis_area();
  my $object = npg_pipeline::analysis::create_psuedo_qseq_custom->new({
    id_run => 1234,
    run_folder => q{123456_Il2_1234},
    runfolder_path => $runfolder_path,
    bustard_path => $bustard_rta,
    recalibrated_path => $gerald_rta,
    verbose => 1,
    conf_path => $conf_path,
    domain => q{test},
  });
  `touch $gerald_rta/s_4_1_qtable.txt`;
  ok($object->control(), q{control found});
  strip_preset_custom_qseqs();

  throws_ok { $object->run(); } qr{Cannot[ ]continue,[ ]as[ ]this[ ]run[ ]has[ ]a[ ]control[ ]lane}, q{run throws ok, as it has a control to be used};
}
{
  set_staging_analysis_area();
  my $object = npg_pipeline::analysis::create_psuedo_qseq_custom->new({
    id_run => 1234,
    run_folder => q{123456_Il2_1234},
    runfolder_path => $runfolder_path,
    bustard_path => $bustard_rta,
    recalibrated_path => $gerald_rta,
    verbose => 1,
    conf_path => $conf_path,
    domain => q{test},
  });
  strip_preset_custom_qseqs();
  `touch $gerald_rta/s_4_1_qtable.txt`;
  ok($object->control(), q{control found});
  `mkdir -p $gerald_rta/Temp/Custom`;
  `touch $gerald_rta/Temp/Custom/s_3_1_001_ub_qseq.txt`;
  `touch $gerald_rta/Temp/Custom/s_3_2_001_ub_qseq.txt`;
  `touch $gerald_rta/Temp/Custom/s_2_1_001_ub_qseq.txt`;
  `touch $gerald_rta/Temp/Custom/s_2_2_001_ub_qseq.txt`;
   

  my @all_qseq_files = $object->_all_qseq_files();
  my $array = [qw{s_2_1_001_qseq s_2_2_001_qseq s_3_1_001_qseq s_3_2_001_qseq}];
  is_deeply(\@all_qseq_files, $array, q{return array is correct});

  lives_ok { $object->run(); } q{run is ok};
  my $hash = {
    q{s_2_1_001_qseq} => 1,
    q{s_2_2_001_qseq} => 1,
    q{s_3_1_001_qseq} => 1,
    q{s_3_2_001_qseq} => 1,
  };
  $object->clear_qseq_custom_files();
  my $all_qseq_custom_files = $object->qseq_custom_files();
  is_deeply($all_qseq_custom_files, $hash, q{qseq_custom_files are all there});
}
{
  set_staging_analysis_area(1);
  my $object = npg_pipeline::analysis::create_psuedo_qseq_custom->new({
    id_run => 1234,
    run_folder => q{123456_Il2_1234},
    runfolder_path => $runfolder_path,
    bustard_path => $bustard_rta,
    recalibrated_path => $gerald_rta,
    verbose => 1,
    conf_path => $conf_path,
    domain => q{test},
  });
  strip_preset_custom_qseqs();
  `touch $gerald_rta/s_4_1_qtable.txt`;
  ok($object->control(), q{control found});
  `mkdir -p $gerald_rta/Temp/Custom`;
  `touch $gerald_rta/Temp/Custom/s_3_1_001_qseq.txt`;
  `touch $gerald_rta/Temp/Custom/s_2_1_001_qseq.txt`;
  `cp t/data/mock_qseqs/1234/s_8_3_0001_qseq.txt $gerald_rta/Temp/Custom/s_2_2_001_qseq.txt`;
  `cp t/data/mock_qseqs/1234/s_8_3_0001_qseq.txt $gerald_rta/Temp/Custom/s_3_2_001_qseq.txt`;
  `cp t/data/mock_qseqs/1234/s_8_2_0001_qseq.txt $bustard_rta/s_2_t_001_qseq.txt`;
  `cp t/data/mock_qseqs/1234/s_8_2_0001_qseq.txt $bustard_rta/s_3_t_001_qseq.txt`;
   

  my @all_qseq_files = $object->_all_qseq_files();
  my $array = [qw{s_2_1_001_qseq s_2_2_001_qseq s_3_1_001_qseq s_3_2_001_qseq}];
  is_deeply(\@all_qseq_files, $array, q{return array is correct});

  lives_ok { $object->run(); } q{run is ok};
  my $hash = {
    q{s_2_1_001_qseq} => 1,
    q{s_2_2_001_qseq} => 1,
    q{s_2_3_001_qseq} => 1,
    q{s_3_1_001_qseq} => 1,
    q{s_3_2_001_qseq} => 1,
    q{s_3_3_001_qseq} => 1,
  };
  $object->clear_qseq_custom_files();
  my $all_qseq_custom_files = $object->qseq_custom_files();
  is_deeply($all_qseq_custom_files, $hash, q{qseq_custom_files are all there});
}
1;