# $Id: useragent.pm 12510 2011-02-01 15:49:31Z ajb $
package t::useragent;
use Moose;
use Test::More;
use English qw(-no_match_vars);
use Carp;

sub BUILD {
  my ($self) = @_;

  if(!$self->has_is_success()) {
    carp qq(Did you mean to leave ua->is_success() unset?);
  }

  return 1;
}

has q{mock} => (isa => q{HashRef}, is => q{rw}, lazy_build => 1);

sub _build_mock {
  return {};
}

has q{last_request} => (isa => q{Any}, is => q{rw});

has q{uri} => (isa => q{Any}, is => q{rw});

sub get {
  my ($self, $uri) = @_;
  $self->uri($uri);
  return $self;
}

sub post {
  my ($self, $uri, %args) = @_;
  $self->uri($uri);
  $self->last_request(\%args);
  return $self;
}

sub request {
  my ($self, $req) = @_;
  $self->uri($req->uri());
  $self->last_request($req);
  return $self;
}

sub requests_redirectable {
  my $self = shift;
  return [];
}

sub content {
  my $self = shift;

  my $test_data = $self->mock()->{$self->uri()};

  my $uri = $self->uri();
  $uri =~ s/\/\z//xms;
  $self->uri($uri);

  my $root = $self->full_root_of_t_directory();
  if(!$test_data) {
    #########
    # try and auto-fetch out of the t/data/ folder
    #
    my ($stpath)  = $self->uri() =~ m{https?://psd\-[^/]+(.*?)$}mx;
    my ($npgpath) = $self->uri() =~ m{https?://intweb.*npg[^/]*?(.*?)$}mx;

    if($npgpath) {
      #########
      # some npg requests have ugly suffixes
      #
      $npgpath =~ s/_xml//mxs;
    }
    if ( $stpath ) {
      $test_data = $root . "t/data/st$stpath.xml";
    } elsif ( $npgpath ) {
      $test_data = $root . "t/data/npg$npgpath.xml";
    }

    if ( ! -e $test_data ) {
      croak qq(No mock data configured for $self->{'uri'});
    }
  }

  my ( $test_path_less_root ) = $test_data =~ /${root}(.*)/xms;
  

  if ( $test_path_less_root =~ m{\At/}xms ) {
    open my $fh, q(<), $test_data or croak qq(Error opening $test_data: $ERRNO);
    local $RS   = undef;
    my $content = <$fh>;
    close $fh;
    return $content;
  }

  return $test_data;
}

has q{is_success} => (isa => q{Bool}, is => q{ro}, predicate => q{has_is_success});

sub response    { my $self = shift; return $self; }

sub status_line { return 'error in t::useragent'; }

sub header {
  return q{application/xml};
}
sub headers {
  my ( $self ) = @_;
  return $self;
}

has q{root_of_t_directory} => (
  isa => q{Str},
  is => q{ro},
  default => q{},
);

sub full_root_of_t_directory {
  my ( $self ) = @_;
  my $return = $self->root_of_t_directory();
  if ( $return ) {
    $return .= q{/};
  }
  return $return;
}


1;
